// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_Receiver_hpp
#define anna_comm_Receiver_hpp

#include <string>

#include <anna/xml/Node.hpp>
#include <anna/comm/Server.hpp>

namespace anna {

namespace comm {

class ClientSocket;
class Server;
class ReceiverFactory;
class Message;

/**
   Modela la clase que gestiona el tratamiento de los mensajes de red de forma independiente para
   cada thread.

   \see Communicator.
   \see ReceiverFactory
*/
class Receiver {
public:
  /**
   * Devuelve el nombre lgico de ste receptor.
   * \return el nombre lgico de ste receptor.
   */
  const std::string& getName() const throw() { return a_name; }

  /**
     Metodo manejador de los mensajes recibidos por el socket. En entornos MT cada socket invocara
     a su Receiver asociado de forma totalmente independiente.

     \param clientSocket Socket cliente por el que ha llegado el mensaje.
     \param message Ultimo mensaje recibido. El bloque de datos recibido ya ha sido
     decodificado aplicando las reglas establecidas por la capa de transporte asociado
     al ClientSocket por el que llega el mensaje.
  */
  virtual void apply(ClientSocket& clientSocket, const Message& message) throw(RuntimeException) = 0;

  /**
   * Mtodo manejador de la notificacin de que el socket va a ser cerrado.
   * param clientSocket Socket cliente que va a ser cerrado.
   */
  virtual void eventBreakConnection(const comm::ClientSocket& clientSocket) throw() {;}

  /**
   * Mtodo manejador de la notificacin de que el socket va a ser cerrado.
   * param clientSocket Socket cliente que va a ser cerrado.
   */
  virtual void eventBreakLocalConnection(const comm::ClientSocket& clientSocket) throw() {;}

  /**
   * Mtodo manejador de la notificacin de que se ha creado una nueva conexion
   * @param server Proceso servidor con el que hemos establecido la conexion.
   */
  virtual void eventCreateConnection(const Server* server) throw() {;}

  /**
      Devuelve una cadena con toda la informacion relevante de este objeto.
      \return una cadena con toda la informacion relevante de este objeto.
   */
  virtual std::string asString() const
  throw() {
    std::string msg("anna::comm::Receiver { Name: ");
    msg += a_name;
    return msg += " }";
  }

  /**
     Devuelve un nodo XML con la informacion referente a este objeto.
     \param parent Nodo XML a partir del cual introducir la informacion.
     \return Un nodo XML con la informacion referente a este objeto.
  */
  virtual xml::Node* asXML(xml::Node* parent) const throw(RuntimeException) {
    xml::Node* node = parent->createChild("comm.Receiver");
    node->createAttribute("Name", a_name);
    return node;
  }

protected:
  /**
     Contructor.
     \param name Nombre logico del receiver.
  */
  Receiver(const char* name) : a_name(name) { ; }

  /**
     Metodo invocado por el ReceiverFactory en el momento de pasar a usar esta instancia.
  */
  virtual void initialize() throw(RuntimeException) { ; }

private:
  std::string a_name;

  friend class ReceiverFactory;

  /**
   * Para poner invocar a #initialize en caso de que se asocie de forma directa el receptor
   * a un anna::comm::ClientSocket.
   */
  friend class ClientSocket;
};

}
}


#endif

