// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_handler_Manager_hpp
#define anna_comm_handler_Manager_hpp

#include <anna/core/Singleton.hpp>
#include <anna/core/util/Recycler.hpp>

#include <anna/comm/handler/Allocator.hpp>
#include <anna/comm/handler/BinderSocket.hpp>
#include <anna/comm/handler/ServerSocket.hpp>
#include <anna/comm/handler/LocalConnection.hpp>
#include <anna/comm/handler/RemoteConnection.hpp>
#include <anna/comm/handler/DatagramSocket.hpp>
#include <anna/comm/handler/ClientSocket.hpp>

namespace anna {

namespace comm {

class Handler;
class Communicator;

namespace handler {

class Manager : public Singleton <Manager> {
public:
  typedef handler::Allocator <handler::BinderSocket> BinderSocketAllocator;
  typedef handler::Allocator <handler::ServerSocket> ServerSocketAllocator;
  typedef handler::Allocator <handler::LocalConnection> LocalConnectionAllocator;
  typedef handler::Allocator <handler::RemoteConnection> RemoteConnectionAllocator;
  typedef handler::Allocator <handler::DatagramSocket> DatagramSocketAllocator;
  typedef handler::Allocator <handler::ClientSocket> ClientSocketAllocator;

  void initialize(Communicator*) throw();

  comm::Handler* createHandler(comm::BinderSocket*) throw(RuntimeException);
  comm::Handler* createHandler(comm::ServerSocket*) throw(RuntimeException);
  comm::Handler* createHandler(comm::LocalConnection*) throw(RuntimeException);
  comm::Handler* createHandler(comm::RemoteConnection*) throw(RuntimeException);
  comm::Handler* createHandler(comm::DatagramSocket*) throw(RuntimeException);
  comm::Handler* createHandler(comm::ClientSocket*) throw(RuntimeException);

  void releaseHandler(Handler*) throw();

private:
  Recycler <handler::BinderSocket, BinderSocketAllocator> a_binderSockets;
  Recycler <handler::ServerSocket, ServerSocketAllocator> a_serverSockets;
  Recycler <handler::LocalConnection, LocalConnectionAllocator> a_localConnections;
  Recycler <handler::RemoteConnection, RemoteConnectionAllocator> a_remoteConnections;
  Recycler <handler::DatagramSocket, DatagramSocketAllocator> a_datagramSockets;
  Recycler <handler::ClientSocket, ClientSocketAllocator> a_clientSockets;

  Manager() {;}
  Manager(const Manager&);

  friend class Singleton <Manager>;
};

}
}
}

#endif

