// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_TextManager_hpp
#define anna_core_util_TextManager_hpp

#include <vector>

#include <anna/core/functions.hpp>
#include <anna/core/mt/Mutex.hpp>

namespace anna {

class TextComposer;

/**
   Gestor de textos con formato.
*/
class TextManager : public Mutex {
public:
  /**
     Constructor.
  */
  TextManager(const char* name);

  /**
     Destructor.
  */
  virtual ~TextManager() { clear(); }

  /**
     Operador de acceso. El compositor solicitado deberia estar creado mediate #create.
     \param composer Indica el numero de compositor al que deseamos acceder.
     \return El compositor de textos.
  */
  TextComposer& operator [](const int composer) throw(RuntimeException) { return find(composer); }

  /**
     Operador de acceso. El compositor solicitado deberia estar creado mediate #create.
     \param composer Indica el numero de compositor al que deseamos acceder.
     \return El compositor de textos.
  */
  const TextComposer& operator [](const int composer) const throw(RuntimeException) { return find(composer); }

  /**
     Crea un nuevo compositor de textos.
     \param composer ndice del compositor.
     \param expression Expresion asociada al compositor que vamos a crear. El numero de variables que hay que
     establecer antes de invocar a TextComposer::apply dependera de este texto.
  */
  void create(const int composer, const char* expression) throw(RuntimeException);

  /**
     Devuelve el compositor de textos asociado al numero recibido como parametro.
     El compositor solicitado deberia estar creado mediate #create.
     \param composer Indica el numero de compositor al que deseamos acceder.
     \return El compositor de textos.
  */
  TextComposer& find(const int composer) throw(RuntimeException);

  /**
     Devuelve el compositor de textos asociado al numero recibido como parametro.
     El compositor solicitado deberia estar creado mediate #create.
     \param composer Indica el numero de compositor al que deseamos acceder.
     \return El compositor de textos.
  */
  const TextComposer& find(const int composer) const throw(RuntimeException);

  /**
     Elimina todos los compositores de texto creados hasta el momento.
  */
  void clear() throw();

  /**
     Metodo que podemos reescribir para crear cualquier clase heredada de TextComposer.
     Por defecto la impletacion sera:
     \code
        return new util::TextComposer (composer, expression);
     \endcode
     \warning La reimplementacion de este metodo nunca deberia devolver NULL.
  */
  virtual TextComposer* createTextComposer(const int composer, const char* expression) throw();

private:
  typedef std::vector <TextComposer*> TextComposerVector;

  const char* a_name;
  TextComposerVector a_composers;

  TextManager(const TextManager&);

  TextComposer* xfind(const int composer) throw();
};

}

#endif

