// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_ZBlock_hpp
#define anna_core_util_ZBlock_hpp

#include <anna/core//DataBlock.hpp>

namespace anna {

/**
   Facilita la compresin y descompresin de bloques de datos.

   Para ms informacin: \see http://zlib.net/manual.html

   Para optimizar el acceso no se ha establecido ningun tipo de proteccion para ejecucion MT.
*/
class ZBlock : public DataBlock {
public:
  /**
   * Nivel de compresin que pueden ser aplicados.
   */
  struct Mode { enum _v { Default = -1, NoCompression, BestSpeed = 1, BestCompression = 9 }; };

  /**
   * Constructor vacio.
   */
  ZBlock() : DataBlock(true) {;}

  /**
   * Comprime el contenido del bloque de datos recibido como parmetro y lo guarda
   * en la estructura interna.
   * \param data Bloque de datos a comprimir.
   * \param mode Modo de compresin realizado.
   * \return El bloque de datos una vez comprimido y con la informacin suficiente para ser descomprimido.
   */
  const DataBlock& compress(const DataBlock& data, const Mode::_v mode = Mode::Default) throw(RuntimeException);

  /**
   * Descomprime el bloque de datos recibido como parmetro.
   * \param zdata Bloque de datos obtenido como resultado de aplicar con ZBlock::compress.
   * \param originalSize Tamao original que tena el buffer antes de ser comprimido.
   * \return El bloque de datos descomprimdo.
   */
  const DataBlock& uncompress(const DataBlock& zdata) throw(RuntimeException);

private:
  ZBlock(const ZBlock&);
};

}

#endif


