// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_comm_Timer_hpp
#define anna_diameter_comm_Timer_hpp

#include <anna/core/RuntimeException.hpp>
#include <anna/core/Allocator.hpp>
#include <anna/timex/Transaction.hpp>


namespace anna {

namespace diameter {

namespace comm {


class Session;
class Response;
class LocalServer;
class TimerManager;


class Timer : public anna::timex::Transaction {

public:
  struct Type { enum _v { ResponseExpiration, SessionUnbind, SessionRecover, LocalServerAttach }; };

  // getters for responses
  Response* getResponse() throw() { return reinterpret_cast <Response*>(getContext()); }
  const Response* getResponse() const throw() { return reinterpret_cast <const Response*>(getContext()); }

  // getters for session (client-session at the moment)
  Session* getSession() throw() { return reinterpret_cast <Session*>(getContext()); }
  const Session* getSession() const throw() { return reinterpret_cast <const Session*>(getContext()); }

  // getters for local server
  LocalServer* getLocalServer() throw() { return reinterpret_cast <LocalServer*>(getContext()); }
  const LocalServer* getLocalServer() const throw() { return reinterpret_cast <const LocalServer*>(getContext()); }

  std::string asString() const throw();

private:
  Timer() {;}


  Type::_v a_type;

  void expire(anna::timex::Engine*) throw(anna::RuntimeException);

  Type::_v getType() const throw() { return a_type; }
  void setType(const Type::_v type)  throw() { a_type = type; }
  static const char* asText(const Type::_v) throw();


  friend class TimerManager;
  friend class anna::Allocator<Timer>;
};

}
}
}

#endif

