// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_io_functions_hpp
#define anna_io_functions_hpp

#include <sys/stat.h>
#include <unistd.h>

#include <anna/config/defines.hpp>
#include <anna/core/RuntimeException.hpp>
#include <anna/core/util/Millisecond.hpp>

namespace anna {

namespace io {

/**
   Grupo de metodos relacionadas con operaciones de entrada/salida
*/
struct functions {
  /**
     Crea el arbol de directorios recibido como parametro.
  */
  static void mkdir(const std::string& path) throw(RuntimeException);

  /**
     Indica si el PATH recibido como parametro existe o no.
     \return Devuelve \em true si el PATH recibido como parametro existe o \em false
     en otro caso.
  */
  static bool exists(const std::string& path) throw(RuntimeException) { return exists(path.c_str()); }

  /**
     Indica si el PATH recibido como parametro existe o no.
     \return Devuelve \em true si el PATH recibido como parametro existe o \em false
     en otro caso.
  */
  static bool exists(const char* path) throw(RuntimeException);

  /**
     Indica si el PATH recibido como parametro es un directorio o no.
     \return Devuelve \em true si el PATH recibido como parametro es un directorio o \em false
     en otro caso.
  */
  static bool isADirectory(const std::string& path) throw(RuntimeException) { return isADirectory(path.c_str()); }

  /**
     Indica si el PATH recibido como parametro es un directorio o no.
     \return Devuelve \em true si el PATH recibido como parametro es un directorio o \em false
     en otro caso.
  */
  static bool isADirectory(const char* path) throw(RuntimeException);

  /**
   * Devuelve el i-nodo asociado al nombre de fichero de recibido como parmetro.
   * \param path Ruta completa del fichero a comprobar.
   * \return El n de i-nodo asociado al fichero
   */
  static ino_t getINode(const std::string& path) throw(RuntimeException) { return getINode(path.c_str()); }

  /**
   * Devuelve el i-nodo asociado al nombre de fichero de recibido como parmetro.
   * \param path Ruta completa del fichero a comprobar.
   * \return El n de i-nodo asociado al fichero
   */
  static ino_t getINode(const char* path) throw(RuntimeException);

  /**
   * Espera a que haya actividad de entrada en el fd recibido como parmetro.
   * \param fd File descriptor sobre el que vamos a comprobar la actividad.
   * \param timeout Nmero mximo de milisegundos que puede quedar a la espera. Si vale 0 no realiza ninguna espera.
   * \return \em true si el proceso detecta actividad o \em false si se cumpli el timeout sin recibir informacin.
   */
  static bool waitInput(const int fd, const Millisecond&) throw(RuntimeException);
};

}
}
#endif
