// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/TraceMethod.hpp>
#include <anna/core/tracing/Logger.hpp>

#include <anna/xml/Node.hpp>

#include <anna/comm/Service.hpp>
#include <anna/comm/Server.hpp>

using namespace std;
using namespace anna;

void comm::Service::attach(comm::Server* server)
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tm(Logger::Local7, "comm::Service", "attach", ANNA_FILE_LOCATION));
  comm::Delivery::add(server);
  server->attach(this);
}

//----------------------------------------------------------------------------------
// (1) Cuando no queda ningun recurso disponible se devuelve una excepcion
//----------------------------------------------------------------------------------
comm::Server* comm::Service::send(comm::Message& message)
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tm(Logger::Local7, "comm::Service", "send", ANNA_FILE_LOCATION));
  comm::Server* tryServer = NULL;
  comm::Server* init = NULL;
  bool stop = false;
  bool sent = false;

  while(stop == false) {
    tryServer = static_cast <Server*>(Delivery::apply());                 // (1)

    try {
      if(init == NULL)
        init = tryServer;
      else if(init == tryServer)
        stop = true;

      tryServer->send(message);
      sent = true;
      break;
    } catch(RuntimeException& ex) {
      ex.trace();
      stop = Delivery::fault(tryServer);
    }
  }

  if(sent == false) {
    string msg(asString());
    msg += " | Service unavailable";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  return tryServer;
}

comm::Server* comm::Service::send(comm::Message* message)
throw(RuntimeException) {
  if(message == NULL)
    throw RuntimeException("comm::Service::send | Cannot send a NULL message", ANNA_FILE_LOCATION);

  return (send(*message));
}

int comm::Service::broadcast(comm::Message& message)
throw() {
  using namespace anna::comm;
  int result = 0;

  for(iterator ii = begin(), maxii = end(); ii != maxii; ii ++) {
    try {
      server(ii)->send(message);
      result ++;
    } catch(RuntimeException& ex) {
      ex.trace();
    }
  }

  LOGDEBUG(
    string msg("comm::Service::broadcast | ");
    msg += asString();
    msg += functions::asText(" | Sents number: ", result);
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  return result;
}

int comm::Service::broadcast(comm::Message* message)
throw() {
  if(message == NULL) {
    Logger::write(Logger::Error, "comm::Service::broadcast | Cannot broadcast a NULL message", ANNA_FILE_LOCATION);
    return 0;
  }

  return broadcast(*message);
}

string comm::Service::asString() const
throw() {
  string result("comm::Service { ");
  result += Delivery::asString();
  result += " | Critical: ";
  result += functions::asString(a_isCritical);
  return result += " }";
}

xml::Node* comm::Service::asXML(xml::Node* parent) const
throw() {
  xml::Node* service = parent->createChild("comm.Service");
  Delivery::asXML(service);
  service->createAttribute("Critical", functions::asString(a_isCritical));
  return service;
}

/*static*/
comm::Server* comm::Service::server(comm::Delivery::iterator& ii)
throw() {
  return static_cast <Server*>(Delivery::resource(ii));
}

/*static*/
const comm::Server* comm::Service::server(comm::Delivery::const_iterator& ii)
throw() {
  return static_cast <const Server*>(Delivery::resource(ii));
}

