// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <pthread.h>

#include <anna/core/mt/Semaphore.hpp>
#include <anna/core/RuntimeException.hpp>
#include <anna/core/functions.hpp>
#include <anna/config/defines.hpp>

using namespace anna;

Semaphore::Semaphore(unsigned int count) :
  Safe() {
#ifdef _MT
  sem_init(&a_id, 0, count);
#endif
}

Semaphore::~Semaphore() {
#ifdef _MT
  sem_destroy(&a_id);
#endif
}

void Semaphore::wait()
throw(RuntimeException) {
#ifdef _MT
  int errorCode;
  anna_signal_shield(errorCode, sem_wait(&a_id));

  if(errorCode != 0)
    throw RuntimeException(std::string("Error doing wait"), errno, ANNA_FILE_LOCATION);

#endif
}

bool Semaphore::tryWait()
throw(RuntimeException) {
  bool result(true);
#ifdef _MT
  int errorCode;

  if((errorCode = sem_trywait(&a_id)) != 0) {
    if(errorCode != EBUSY)
      throw RuntimeException(std::string("Error trying wait"), errno, ANNA_FILE_LOCATION);
    else
      result = false;
  }

#endif
  return result;
}

void Semaphore::signal()
throw(RuntimeException) {
#ifdef _MT
  int errorCode = sem_post(&a_id);

  if(errorCode != 0)
    throw RuntimeException(std::string("Error doing post"), errno, ANNA_FILE_LOCATION);

#endif
}

std::string Semaphore::asString() const
throw() {
  std::string msg("anna::Semaphone { Id: ");
  msg += functions::asHexString(anna_ptrnumber_cast(&a_id));
  msg += " | Value: ";
#ifdef _MT
  int value = 0;
  sem_getvalue(&(const_cast <Semaphore*>(this)->a_id), &value);
  msg += functions::asString(value);
#else
  msg += "<none>";
#endif
  return msg += " }";
}

