// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/util/EncodedData.hpp>
#include <anna/core//DataBlock.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

#include <anna/core/util/Tokenizer.hpp>

using namespace std;
using namespace anna;

void EncodedData::initialize(const xml::Node* parent)
throw(RuntimeException) {
  if(fromBCD(parent->find("Key1")->getAttribute("Value")->getValue(), a_value) != sizeof(DES_key_schedule))
    throw RuntimeException("anna::EncodedData::initialize | Key1 no valida", ANNA_FILE_LOCATION);

  anna_memcpy(&a_skey[0], a_value.getData(), sizeof(DES_key_schedule));

  if(fromBCD(parent->find("Key2")->getAttribute("Value")->getValue(), a_value) != sizeof(DES_key_schedule))
    throw RuntimeException("anna::EncodedData::initialize | Key2 no valida", ANNA_FILE_LOCATION);

  anna_memcpy(&a_skey[1], a_value.getData(), sizeof(DES_key_schedule));

  if(fromBCD(parent->find("Key3")->getAttribute("Value")->getValue(), a_value) != sizeof(DES_key_schedule))
    throw RuntimeException("anna::EncodedData::initialize | Key3 no valida", ANNA_FILE_LOCATION);

  anna_memcpy(&a_skey[2], a_value.getData(), sizeof(DES_key_schedule));

  if(fromBCD(parent->find("IVector")->getAttribute("Value")->getValue(), a_value) != sizeof(DES_cblock))
    throw RuntimeException("anna::EncodedData::initialize | IVector no valida", ANNA_FILE_LOCATION);

  anna_memcpy(&a_iv, a_value.getData(), sizeof(DES_cblock));
  const xml::Node* node = parent->find("DataBlock");
  a_realSize = node->getAttribute("Length")->getIntegerValue();
  fromBCD(node->getAttribute("Value")->getValue(), a_value);
}

xml::Node* EncodedData::asXML(xml::Node* parent) const
throw(RuntimeException) {
  parent->createAttribute("Mode", "DES3");
  xml::Node* node;
  string aux;
  DataBlock key1((char*) &a_skey[0], sizeof(DES_key_schedule), false);
  parent->createChild("Key1")->createAttribute("Value", asBCD(key1, aux));
  DataBlock key2((char*) &a_skey[1], sizeof(DES_key_schedule), false);
  parent->createChild("Key2")->createAttribute("Value", asBCD(key2, aux));
  DataBlock key3((char*) &a_skey[2], sizeof(DES_key_schedule), false);
  parent->createChild("Key3")->createAttribute("Value", asBCD(key3, aux));
  DataBlock iv((char*) &a_iv, sizeof(DES_cblock), false);
  parent->createChild("IVector")->createAttribute("Value", asBCD(iv, aux));
  node = parent->createChild("DataBlock");
  node->createAttribute("Length", a_realSize);
  node->createAttribute("Value", asBCD(a_value, aux));
  return parent;
}

const std::string& EncodedData::asBCD(const DataBlock& source, string& result)
throw() {
  result = "";
  const char* data = source.getData();
  int c;
  int byte;

  for(int len = 0, maxlen = source.getSize(); len < maxlen; len ++, data ++) {
    c = *data;
    byte = (c & 0xf0) >> 4;
    result += (byte <= 9) ? (byte + '0') : (byte - 0x0a + 'a');
    byte = c & 0x0f;
    result += (byte <= 9) ? (byte + '0') : (byte - 0x0a + 'a');
  }

  return result;
}

int EncodedData::fromBCD(const string& source, DataBlock& result)
throw() {
  result.clear();
  const char* data = source.c_str();
  int len = 0, maxlen = source.length();
  int c;
  int byte;

  while(len < maxlen) {
    c = *data ++;
    byte = (c <= '9') ? (c - '0') : (c - 'a' + 0x0a);
    byte <<= 4;

    if(++ len < maxlen) {
      c = *data ++;
      byte |= (c <= '9') ? (c - '0') : (c - 'a' + 0x0a);
    }

    result += char(byte);
    len ++;
  }

  return result.getSize();
}



