// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/diameter/stack/Command.hpp>
#include <anna/diameter/stack/Avp.hpp>
#include <anna/diameter/functions.hpp>
#include <anna/diameter/stack/Dictionary.hpp>

#include <anna/core/functions.hpp>
#include <anna/xml/xml.hpp>

//using namespace diameter::stack;


//------------------------------------------------------------------------------
//-------------------------------------------------------- Command::addAvpRule()
//------------------------------------------------------------------------------
void anna::diameter::stack::Command::addAvpRule(const AvpRule & avpRule) throw(anna::RuntimeException) {
  if(avpRule.isFixed()) {
    if(!a_allowFixedRule) {
      std::string s_ex = anna::functions::asString("Incorrect position for fixed avp rule '<%s>' within command '%s'", avpRule.getAvpName().c_str(), getName().c_str());
      s_ex += ". Fixed avp rules must be located at the beginning";
      throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
    }
  } else a_allowFixedRule = false;

  //a_avprules[(avpRule.getAvp())->getId()] = avpRule;

  // Restriction for redefinition (at this same level) of two rules for the same avp:
  if(isChild(avpRule.getId())) {
    std::string s_ex = anna::functions::asString("Cannot add two rules for avp '%s', at the same level within command '%s'", avpRule.getAvpName().c_str(), getName().c_str());
    throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
  }

  a_avprules[a_avprulePosition++] = avpRule;
}


//------------------------------------------------------------------------------
//----------------------------------------------------------- Command::isChild()
//------------------------------------------------------------------------------
bool anna::diameter::stack::Command::isChild(const AvpId & avpId) const throw() {
  for(const_avprule_iterator it = avprule_begin(); it != avprule_end(); it++)
    if(avpId == ((*it).second.getId()))
      return true;

  return false;
}


//------------------------------------------------------------------------------
//---------------------------------------------------------- Command::asString()
//------------------------------------------------------------------------------
std::string anna::diameter::stack::Command::asString(void) const throw() {
  std::string trace;
  //trace = "Command '";
  trace = "'";
  trace += a_name;
  trace += "' ";
  trace += anna::diameter::functions::commandIdAsPairString(a_id);
  trace += "\n";

  if(isEmpty()) {
    trace += DICTIONARY_AVPRULE_TAB;
    trace += "No Avp rules defined\n";
  } else {
    for(const_avprule_iterator it = avprule_begin(); it != avprule_end(); it++) {
      // Align qualifier
      std::string qual = (*it).second.getQual();
      int NumberOfSpaces = strlen(DICTIONARY_AVPRULE_TAB) - qual.size();

      for(int k = 0; k < NumberOfSpaces; k++) trace += " ";

      trace += (*it).second.asString();
      trace += "\n";
    }
  }

  return (trace);
}


//------------------------------------------------------------------------------
//------------------------------------------------------------- Command::asXML()
//------------------------------------------------------------------------------
anna::xml::Node* anna::diameter::stack::Command::asXML(anna::xml::Node* parent) const throw() {
//   <!ELEMENT command (avprule*)>
//   <!ATTLIST command name CDATA #REQUIRED code CDATA #REQUIRED type (Request | Answer) #REQUIRED>
  anna::xml::Node* result = parent->createChild("command");
  result->createAttribute("name", a_name);
  result->createAttribute("code", anna::functions::asString(a_id.first));
  result->createAttribute("type", anna::functions::asString(a_id.second ? "Request" : "Answer"));

  for(const_avprule_iterator it = avprule_begin(); it != avprule_end(); it++)
    (*it).second.asXML(result);

  return result;
}

